package org.xbib.interlibrary.slnp.kobv;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Logger;
import org.xbib.interlibrary.api.order.tracking.Event;
import org.xbib.interlibrary.api.order.tracking.EventHandler;
import org.xbib.interlibrary.slnp.SlnpClient;
import org.xbib.settings.Settings;

public class OrderTracker implements org.xbib.interlibrary.api.order.tracking.OrderTracker {

    private static final Map<String, String> domainMap = new HashMap<>() {{
        put("DE-576", "SWB");
        put("DE-601", "GBV");
        put("DE-602", "KOBV");
        put("DE-603", "HEBIS");
        put("DE-604", "BVB");
        put("DE-605", "HBZ");
    }};

    private static final Map<String, String> codeMap = new HashMap<>() {{
        put("Cancel", "NOTSUPPLIED");
        put("CancelAFL", "NOTSUPPLIED");
        put("Expire", "EXPIRE");
        put("Failure", "ERROR");
        put("Finish", "END");
        put("Zfl2Lws", "PREPARING");
        put("ZFl2LwS", "PREPARING");
        put("Lsw2Zfl", "ROUTING");
        put("Lsw2ZFL", "ROUTING");
        put("LwS2ZFl", "ROUTING");
        put("OrderIn", "INCOMING");
        put("PrintRL", "PROCESSED");
        put("Return", "RETURN");
        put("Shipped", "SHIP");
        put("Sign", "SIGN");
        put("WorkOut", "INPROCESS");
        put("Quit", "QUIT");
    }};

    private static final DateTimeFormatter formatter = new DateTimeFormatterBuilder()
            .parseCaseInsensitive()
            .appendPattern("dd.MM.yyyy HH:mm:ss")
            .toFormatter(Locale.GERMAN);

    private static final Map<String, String> infoMap = new HashMap<>() {{
        put("PREPARING", "Konfiguration wird eingelesen");
        put("ROUTING", "Leitweg wird festgelegt");
        put("INCOMING", "Eingang der Bestellung");
        put("INPROCESS", "in Bearbeitung");
        put("PROCESSED", "bearbeitet");
        put("PENDING", "in Vorbereitung");
        put("SHIP", "Versendet");
        put("IDLE", "Unbekannt");
        put("NOTSUPPLIED", "keine Lieferung");
        put("CANCEL", "Abgesagt");
        put("END", "Beendet");
        put("ERROR", "Fehler");
        put("EXPIRE", "Abgelaufen");
        put("FORWARD", "Weitergeleitet");
        put("RETURN", "Zurückgesendet");
        put("SIGN", "Signieren");
        put("QUIT", "Quittiert");
    }};

    private SlnpClient client;

    private String host;

    private int port;

    public OrderTracker() {
    }

    @Override
    public String getISIL() {
        return "DE-602";
    }

    @Override
    public void init(Settings settings) {
        this.client = new SlnpClient();
        this.host = settings.get("host");
        Objects.requireNonNull(host);
        this.port = settings.getAsInt("port", 8881);
    }

    @Override
    public void track(String orderNumber, EventHandler handler) throws IOException {
        String cmd = "SLNPFLHistory\nBestellId:" + orderNumber.trim() + "\nSLNPEndCommand";
        List<String> responses = client.send(host, port, cmd);
        Logger.getAnonymousLogger().info("response = " + responses);
        if (responses == null || responses.isEmpty()) {
            return;
        }
        collectEvents(responses, handler);
    }

    @Override
    public void track(String domain, String requesterId, String transactionId, EventHandler handler) throws IOException {
        String mappedDomain = domainMap.getOrDefault(domain, domain);
        Objects.requireNonNull(mappedDomain);
        Objects.requireNonNull(transactionId);
        String cmd = "SLNPFLHistory\nSigelNB:" + requesterId + "\nPfl2Afl:" + transactionId.trim() + "\nSLNPEndCommand";
        List<String> responses = client.send(host, port, cmd);
        Logger.getAnonymousLogger().info("response = " + responses);
        if (responses == null || responses.isEmpty()) {
            return;
        }
        collectEvents(responses, handler);
    }

    private void collectEvents(List<String> responses, EventHandler handler) {
        Map<String, String> event = new HashMap<>();
        for (String response : responses) {
            int pos = response.indexOf(' ');
            String numCode = pos >= 0 ? response.substring(0, pos) : response;
            String textCode = pos >= 0 ? response.substring(pos + 1) : "";
            pos = textCode.indexOf(':');
            String key = pos >= 0 ? textCode.substring(0, pos) : textCode;
            String value = pos >= 0 ? textCode.substring(pos + 1) : "";
            switch (numCode) {
                case "602": {
                    if (!event.isEmpty()) {
                        emit(event, handler);
                    }
                    event = new HashMap<>();
                    break;
                }
                case "603": {
                    switch (key) {
                        case "LfdNr":
                            event.put("pos", value.trim());
                            break;
                        case "Vorgang":
                            event.put("code", value.trim());
                            break;
                        case "Ausgang":
                            // ignore
                            break;
                        case "Datum":
                            event.put("time", value.trim());
                            break;
                        case "Uhrzeit":
                            String time = event.get("time");
                            event.put("time", time + " " + value);
                            break;
                        case "Sigel":
                            if (value.startsWith("E#")) {
                                value = value.substring(2);
                            }
                            event.put("isil", value);
                            break;
                        case "WhyNot":
                            event.put("info", value.trim());
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        if (!event.isEmpty()) {
            emit(event, handler);
        }
    }

    private void emit(Map<String, String> map, EventHandler handler) {
        Event event = new Event() {
            @Override
            public Integer getPos() {
                return map.containsKey("pos") ? Integer.parseInt(map.get("pos")) : null;
            }

            @Override
            public LocalDateTime getTime() {
                String time = map.get("time");
                if (time != null && !time.isEmpty() && !time.isBlank()) {
                    return LocalDateTime.parse(time, formatter);
                } else {
                    return null;
                }
            }

            @Override
            public String getIsil() {
                return map.get("isil");
            }

            @Override
            public String getCode() {
                return codeMap.get(map.get("code"));
            }

            @Override
            public String getInfo() {
                return map.containsKey("info") ? map.get("info") : infoMap.get(getCode());
            }
        };
        if (handler != null) {
            handler.onEvent(event);
        }
    }
}
